#include "crapfile.h"

#ifdef PIXELTOASTER

#include <string.h>

CrapFile::CrapFile(const char *filename, const char *mode)
{
	mCanWrite = mode[0] == 'w';
	if (strstr(filename, "ms0:/") == filename)
	{
		filename += strlen("ms0:/");
	}
	mFile = fopen(filename, mCanWrite ? "wb" : "rb");
}

CrapFile::~CrapFile()
{
	if (mFile) fclose(mFile);
}

bool CrapFile::IsOpen() const
{
	return mFile != 0;
}

void CrapFile::Read(void* buf, int numBytes)
{
	if (!mCanWrite)
	{
		fread(buf, numBytes, 1, mFile);
	}
}

void CrapFile::Write(void* buf, int numBytes)
{
	if (mCanWrite)
	{
		fwrite(buf, numBytes, 1, mFile);
	}
}

CrapDir::CrapDir(const char *dir) : mNumEntries(0)
{
	WIN32_FIND_DATA FindFileData;
	HANDLE hFind = INVALID_HANDLE_VALUE;
	char DirSpec[MAX_PATH];

	strcpy(DirSpec, dir);
	strcat(DirSpec, "\\*");

	hFind = FindFirstFile(DirSpec, &FindFileData);

	if (hFind == INVALID_HANDLE_VALUE) 
	{
		mNumEntries = 0;
	} 
	else 
	{
		if (strcmp(FindFileData.cFileName, ".") != 0) strcpy(mEntries[mNumEntries++], FindFileData.cFileName);
		while (FindNextFile(hFind, &FindFileData) != 0) 
		{
			if (strcmp(FindFileData.cFileName, ".") != 0) strcpy(mEntries[mNumEntries++], FindFileData.cFileName);
		}
		FindClose(hFind);
	}
}

int CrapDir::NumEntries() const { return mNumEntries; }
const char *CrapDir::GetEntry(int n) const { return mEntries[n]; }


#else

#define O_RDONLY    0x0001 
#define O_WRONLY    0x0002 
#define O_RDWR      0x0003 
#define O_NBLOCK    0x0010 
#define O_APPEND    0x0100 
#define O_CREAT     0x0200 
#define O_TRUNC     0x0400 
#define O_NOWAIT    0x8000 

#define SEEK_CUR    1
#define SEEK_END    2
#define SEEK_SET    0

extern "C"
{
	typedef struct sceDirent
	{
		unsigned int unk0;
		unsigned int type;
		unsigned int size;
		unsigned int unk[19];
		char name[0x108];
	};

	int sceIoOpen(const char* file, int mode); 
	void sceIoClose(int fd); 
	int sceIoRead(int fd, void *data, int size); 
	int sceIoWrite(int fd, void *data, int size); 
	int sceIoLseek(int fd, long long offset, int whence); 
	int sceIoRemove(const char *file); 
	int sceIoMkdir(const char *dir, int mode); 
	int sceIoRmdir(const char *dir); 
	int sceIoRename(const char *oldname, const char *newname); 
	int sceIoDopen(const char *fn);
	int sceIoDread(int fd, struct sceDirent *de);
	void sceIoDclose(int fd); 
}

CrapFile::CrapFile(const char *filename, const char *mode)
{
	int m = mode[0] == 'w' ? (O_WRONLY|O_CREAT) : O_RDONLY;
	mFD = sceIoOpen(filename, m);

	// have to do this here rather than on demand because I can't get the current
	// file position to restore it after moving to the end.
	if (mode[0] == 'w')
	{
		mLength = 0;
	}
	else
	{
		mLength = sceIoLseek(mFD, 0, SEEK_END);
		sceIoClose(mFD);
		mFD = sceIoOpen(filename, m);
	}
}

CrapFile::~CrapFile()
{
	if (mFD) sceIoClose(mFD);
}

bool CrapFile::IsOpen() const
{
	return mFD != 0;
}

bool CrapFile::Read(void* buf, int numBytes)
{
	int readBytes = sceIoRead(mFD, buf, numBytes);
	if (readBytes != numBytes) return false;
	return true;
}

void CrapFile::Write(void* buf, int numBytes)
{
	sceIoWrite(mFD, buf, numBytes);
}

long CrapFile::Length()
{
	return mLength;
}

CrapDir::CrapDir(const char *dir) : mNumEntries(0)
{
	int fd = sceIoDopen(dir);
	if (fd == -1) return;

	sceDirent de;

	while (sceIoDread(fd, &de) != -1)
	{
		if (!(de.name[0] == '.' && de.name[1] == 0))
		{
			char *d = mEntries[mNumEntries++];
			for (char *p = de.name; *p;) *d++ = *p++;
		}
	}

	sceIoDclose(fd);
}

int CrapDir::NumEntries() const { return mNumEntries; }
const char *CrapDir::GetEntry(int n) const { return mEntries[n]; }

#endif
